/*
 * $Id: DccmConnection.cs 283 2007-06-05 13:55:28Z lloydd $
 */

using System;
using System.IO;
using System.Net;
using System.Text;
using System.Net.Sockets;
using System.Threading;
using System.Collections;
using System.Collections.Generic;
using System.Reflection;
using Mono.Unix;
using Mono.GetOptions;
using NDesk.DBus;
using org.freedesktop.DBus;
using org.wimo.dccm;

namespace Dccm
{
   public class DccmConnection
   {
      Device device;
      bool waiting_for_password;
      bool device_added;
      NetworkStream stream;
      TcpClient client;
      string ip_addr;
      byte device_key;
      
      public DccmConnection (TcpClient client)
      {
         this.client = client;
         this.stream = client.GetStream ();
         
         device = null;
         waiting_for_password = false;
         device_added = false;
         device_key = 0;
         ip_addr = ((IPEndPoint) client.Client.RemoteEndPoint).Address.ToString ();
      }
      
      public void Handler ()
      {
         if (MainClass.Opt.Debug)
            Console.WriteLine ("Starting to process connection from {0}", ip_addr);

         try {
            
            while (Process ()) {
               
               if (!waiting_for_password) {
		  
                  Thread.Sleep (3000);

                  Ping ();
               }
            }
         }
         catch (Exception e) {
            
            Console.Error.WriteLine ("Exception: {0}", e.Message);
         }

         if (device != null) {
            
            MainClass.Manager.Disconnected (device);
            
            if (device_added)
               MainClass.Manager.Remove (device);
         }
         
         if (MainClass.Opt.Debug)
            Console.WriteLine ("Connection from {0} closed", ip_addr);
      }
      
      public bool SendPassword (string psw, uint key)
      {
         byte[] encoded = new byte [2 * (psw.Length + 1)];
         
         for (int i = 0; i < psw.Length; i++)
            encoded [i * 2] = (byte) psw [i];

         for (int i = 0; i < encoded.Length; i++)
            encoded [i] = (byte)(encoded [i] ^ key);
         
         byte[] len = BitConverter.GetBytes ((ushort) encoded.Length);
         
         stream.Write (len, 0, len.Length);
         stream.Write (encoded, 0, encoded.Length);

         len = new byte [2];

         if (stream.Read (len, 0, len.Length) != len.Length)
            return false;

         return len [0] == 1;
      }
      
      public void Ping ()
      {
         if (MainClass.Opt.Debug)
            Console.WriteLine ("Sending ping");
         
         byte[] ping = BitConverter.GetBytes ((uint) 0x12345678);
         
         stream.Write (ping, 0, ping.Length);
      }
      
      public bool Process ()
      {
         byte[] buf = null;
         int length = 0;

         buf = new byte [4];
         if (stream.Read (buf, 0, buf.Length) != 4)
            return false;
         
         length = BitConverter.ToInt32 (buf, 0);
         
         if (MainClass.Opt.Debug)
            Console.WriteLine ("Receiving data length: {0}", length);
         
         if (length == 0) {
            // INIT
            if (MainClass.Opt.Debug)
               Console.WriteLine ("Communication initialization");
            
         } else if (length == 0x12345678) {
            // PING
            if (MainClass.Opt.Debug)
               Console.WriteLine ("Received ping");
            
         } else if (length >= 512) {
            // password request
            if (MainClass.Opt.Debug)
               Console.WriteLine ("Received password request");
            
            device_key = (byte) (length & 0xff);
            waiting_for_password = true;
	    
         } else if (length >= 0x24) {
            // header information
            if (MainClass.Opt.Debug)
               Console.WriteLine ("Received header information");
            
            buf = new byte [length];
            if (stream.Read (buf, 0, buf.Length) != length)
               return false;
            
            if ((device = ProcessInfo (buf)) != null) {
               
               MainClass.Manager.Add (device);
               device_added = true;
               
               MainClass.Manager.Connected (device);
               
               if (waiting_for_password) {
                  
                  if (MainClass.Opt.Debug)
                     Console.WriteLine ("Signaling password request");
                  
                  device.PasswordProvided = false;
                  
                  MainClass.Manager.PasswordRequested (device);
                  
                  int cnt = 15*4;
                  
                  while (!device.PasswordProvided)
                     if (cnt-- > 0)
                        Thread.Sleep (250);
                     else
                        break;
                  
                  if (device.PasswordProvided) {
                     
                     if (MainClass.Opt.Debug)
                        Console.WriteLine ("Password provided");
                     
                     if (SendPassword (device.Password, device.Key)) {
                        
                        if (MainClass.Opt.Debug)
                           Console.WriteLine ("Password accepted");
                  
                        Thread.Sleep (250); // gives some time to the device
                        
                        MainClass.Manager.ConnectionEstablished (device);
                        
                        client.NoDelay = true;
                        client.ReceiveTimeout = 3000;
                     } else {
                        
                        if (MainClass.Opt.Debug)
                           Console.WriteLine ("Password not accepted");
                        
                        MainClass.Manager.WrongPassword (device);
                        
                        return false;
                     }
                  } else
                     if (MainClass.Opt.Debug)
                        Console.WriteLine ("Password not provided");
                     
                  waiting_for_password = false;
               } else {
                
                  MainClass.Manager.ConnectionEstablished (device);
                  
                  client.NoDelay = true;
                  client.ReceiveTimeout = 3000;
               }
            }
         }

         return true;
      }
      
      public Device ProcessInfo (byte[] buf)
      {
         if (MainClass.Opt.Debug)
            WriteBuf ("ProcessInfoData", buf, buf.Length);

         uint os_major = buf [4];
         uint os_minor = buf [5];
         uint version = BitConverter.ToUInt16 (buf, 6);
         uint cpu_type = BitConverter.ToUInt16 (buf, 8);
         uint current_partner_id = BitConverter.ToUInt32 (buf, 0x10);
         uint id = BitConverter.ToUInt32 (buf, 0x14);
         string name = GetStringAt (buf, 0x18);
         string platform_name = GetStringAt (buf, 0x1c);
         string model_name = GetStringAt (buf, 0x20);
         
         Device dev = new Device ();
         dev.OsVersionMajor = os_major;
         dev.OsVersionMinor = os_minor;
         dev.Version = version;
         dev.CpuType = cpu_type;
         dev.CurrentPartnerId = current_partner_id;
         dev.Id = id;
         dev.Name = name;
         dev.PlatformName = platform_name;
         dev.ModelName = model_name;
         dev.DeviceIP = ip_addr;
         dev.DeviceRapiIP = ip_addr;
         dev.DeviceRapiPort = 990;
         dev.Key = device_key;
         dev.PasswordProtected = waiting_for_password;
         
         return dev;
      }
      
      public string GetStringAt (byte[] buf, int pos)
      {
         if (pos >= buf.Length)
            return "";
         
         int ofs = BitConverter.ToInt32 (buf, pos);
         int end = ofs;
         
         while ((end + 1) < buf.Length)
            if (buf [end] == 0 && buf [end + 1] == 0)
               break;
            else
               end += 2;
            
         return Encoding.Unicode.GetString (buf, ofs, end - ofs);
      }
      
      public void WriteBuf (string text, byte[] data, int size)
      {
         Console.WriteLine ("{0}: ", text);
         for (int i = 0; i < size; i++)
            Console.Write ("{0} {1}", data [i].ToString ("X2"), ((i+1) % 16 == 0 ? "\n" : ""));
         Console.WriteLine ();
         
         Console.WriteLine ("{0}: ", text);
         for (int i = 0; i < size; i++)
            Console.Write ("{0} {1}", (char) data [i], ((i+1) % 16 == 0 ? "\n" : ""));
         Console.WriteLine ();
      }
   }
}
