/*
 * $Id: Detector.cs 287 2007-06-06 07:05:28Z lloydd $
 */
 
using System;
using System.IO;
using System.Net;
using System.Net.Sockets;
using System.Diagnostics;
using System.Collections;
using System.Collections.Generic;
using System.Threading;
using Mono.Unix;
using NDesk.DBus;
using org.wimo.dccm;

namespace Dccm
{
   public class Detector
   {
      Hal.Manager HalMgr;

      public Detector ()
      {
         HalMgr = new Hal.Manager ();
         HalMgr.DeviceAdded += new Hal.DeviceAddedHandler (DeviceAdded);
      }
      
      public void DeviceAdded (object o, Hal.DeviceAddedArgs args)
      {
         if (MainClass.Opt.Debug)
            Console.WriteLine ("Added device udi:{0}", args.Udi);
         
         // WM2003 SE and older
         if (IsWM3 (args.Device)) {
               
            if (MainClass.Opt.Debug)
               Console.WriteLine ("Found added WM2003SE device or older");
            
            RunScript (Defines.SerialScript, "add", args.Udi);
         }
         
         // WM5 and newer
         if (IsWM5 (args.Device)) {

            if (MainClass.Opt.Debug)
               Console.WriteLine ("Found added WM5 device or newer");
            
            if (RunScript (Defines.RNDISScript, "add", args.Udi) == 0)
               StartTriggerConnection ();
         }
      }
      
      public int RunScript (string script, string type, string udi)
      {
         int ret = -1;
         string cmd = script;
         
         if (cmd.Length > 0)
            ret = ExecProcess (cmd, "{0} {1}", type, udi);
         
         if (ret != 0)
            Console.Error.WriteLine (Catalog.GetString ("Error: exitcode {0} for \"{1}\""),
               ret, string.Format ("{0} {1} {2}", cmd, type, udi));
            
         return ret;
      }

      public static int ExecProcess (string command, string format, params object[] args)
      {
         string arg = string.Format (format, args);
         
         Process process = Process.Start (command, arg == null ? "" : arg);
         
         process.WaitForExit ();
         
         return process.ExitCode;
      }
      
      public bool IsWM3 (Hal.Device dev)
      {
         try {
            
            if (dev.QueryCapability ("pda"))
               if (dev.QueryCapability ("serial"))
                  return true;
         }
         catch (Exception e) {
            
            Console.Error.WriteLine ("Failed to detect attached WM2003SE and older. {0}", e.Message);
         }
         
         return false;
      }
      
      public bool IsWM5 (Hal.Device dev)
      {
         string parent = "";
         string product = "";
         
         try {
            
            if (dev.QueryCapability ("net"))
               while ((parent = dev ["info.parent"]) != null && parent.Length > 0) {
                  
                  dev = new Hal.Device (parent);
                  
                  if ((product = dev ["info.product"]) != null && product == "USB RNDIS PDA")
                     return true;
               }
         }
         catch (Exception e) {
            
            Console.Error.WriteLine ("Failed to detect attached WM5 and newer. {0}", e.Message);
         }
         
         return false;
      }

      public void StartTriggerConnection ()
      {
         if (MainClass.Opt.Debug)
            Console.WriteLine ("Starting trigger connection thread");
         
         new Thread (new ThreadStart (TriggerConnectionHandler)).Start ();
      }
      
      public void TriggerConnectionHandler ()
      {
         for (int i = 0; i < 10; i++) {

            TriggerConnection ();
            
            Thread.Sleep (1000);
         }
      }
      
      public static void TriggerConnection ()
      {
         byte[] req = new byte [1] { 0x7f };
         
         IPEndPoint device = new IPEndPoint (
                                    IPAddress.Parse ("169.254.2.1"),
                                    5679);

         Socket client = new Socket (AddressFamily.InterNetwork,
                              SocketType.Dgram, ProtocolType.Udp);

         if (MainClass.Opt.Debug)
            Console.WriteLine ("Triggering WM5 device DCCM connection");

         client.SendTo (req, req.Length, SocketFlags.None, device);
         client.Close ();
      }
   }
}
