/*
 * $Id: RapiServer.cs 287 2007-06-06 07:05:28Z lloydd $
 */
 
using System;
using System.IO;
using System.Net;
using System.Net.Sockets;
using System.Threading;
using System.Collections;
using System.Collections.Generic;
using System.Reflection;
using Mono.Unix;
using Mono.GetOptions;
using NDesk.DBus;
using org.freedesktop.DBus;
using org.wimo.dccm;

namespace Dccm
{
   public class DeviceRequest
   {
      public TcpClient DeviceSck;
      public uint ReqId;
      
      public DeviceRequest (TcpClient device_sck, uint req_id)
      {
         DeviceSck = device_sck;
         ReqId = req_id;
      }
   }
   
   public class RapiServer
   {
      static Dictionary<string,Dictionary<uint,TcpClient>> request_device;
      static TcpListener server;
      static int next_rapi_port;

      public RapiServer ()
      {
         ResetRapiPort ();
         
         request_device = new Dictionary<string,Dictionary<uint,TcpClient>> ();
         
         server = new TcpListener (IPAddress.Any, 990);
         
         server.Start ();
         
         new Thread (new ThreadStart (Handler)).Start ();
      }
      
      public static void PushRequest (string ip, TcpClient client, uint req_id)
      {
         if (request_device.ContainsKey (ip))
            request_device [ip][req_id] = client;
      }
      
      public static void Handler ()
      {
         TcpClient sck = null;
         TcpClient client_sck = null;
         RapiForwarder fwd = null;
         string ip = "";
         uint req_id = 0;
         RapiConnection conn = null;
         
         while (true) {
            
            if ((sck = server.AcceptTcpClient ()) == null)
               continue;
   
            ip = ((IPEndPoint) sck.Client.RemoteEndPoint).Address.ToString ();

            if (MainClass.Opt.Debug)
               Console.WriteLine ("Accepted RAPI connection from {0}", ip);
            
            if (!ReadReq (sck, out req_id)) {
               
               Console.Error.WriteLine ("Failed to receive request id from new RAPI connection");
               
               sck.Close ();
               
               continue;
            }

            if (MainClass.Opt.Debug)
               Console.WriteLine ("Request id: {0}", req_id);
            
            if (!request_device.ContainsKey (ip) && req_id > 0) {
               
               sck.Close ();
               
               continue;
            }
            
            if (req_id == 0) {
               // new device connect
               conn = null;
               
               ResetRapiPort ();
               
               while (true) {
               
                  try {
                     conn = new RapiConnection (sck, "127.0.0.1", GetNextRapiPort ());
   
                     request_device [ip] = new Dictionary<uint,TcpClient> ();
                     
                     conn.Start ();
                     
                     break;
                  }
                  catch (SocketException e) {
                     
                     if (e.ErrorCode == 10048)
                        continue;
                     
                     Console.Error.WriteLine ("Failed to initialize processing of new RAPI connection. {0}:{1}", e.ErrorCode, e.Message);
                  }
                  catch (Exception e) {
                     
                     Console.Error.WriteLine ("Failed to initialize processing of new RAPI connection. {0}", e.Message);
                  }
                  
                  break;
               }
               
               if (conn != null)
                  continue;
            } else
               if (request_device.ContainsKey (ip)) {
                  // new client connection
                  if (request_device [ip].ContainsKey (req_id)) {
                     
                     client_sck = request_device [ip][req_id];
                     
                     request_device [ip].Remove (req_id);
                     
                     if (client_sck != null) {
                        
                        fwd = new RapiForwarder (sck, client_sck);
                        fwd.Start ();
                        
                        continue;
                     }
                  }
               }
               
            sck.Close ();
         }
      }

      public static bool ReadReq (TcpClient sck, out uint req_id)
      {
         byte[] buf = new byte [sizeof (uint)];
         
         req_id = 0;
         
         if (sck.GetStream ().Read (buf, 0, buf.Length) != buf.Length)
            return false;
         
         req_id = BitConverter.ToUInt32 (buf, 0);
         
         return true;
      }
      
      public static void ResetRapiPort ()
      {
         next_rapi_port = 10000;
      }
      
      public static int GetNextRapiPort ()
      {
         return next_rapi_port++;
      }
   }
}
