/*
 * $Id: dccm.cs 283 2007-06-05 13:55:28Z lloydd $
 */
 
using System;
using System.IO;
using System.Net;
using System.Net.Sockets;
using System.Threading;
using System.Collections;
using System.Collections.Generic;
using System.Reflection;
using Mono.Unix;
using Mono.GetOptions;
using NDesk.DBus;
using org.freedesktop.DBus;
using org.wimo.dccm;

namespace Dccm
{
   public class MainClass
   {
      public static DeviceManager Manager;
      public static DaemonOptions Opt;

      public static void PrintDevice (Device dev, string path, string event_type)
      {
         Console.WriteLine ("---[ {0} ]---", event_type);
         Console.WriteLine ("Path: {0}", dev != null ? dev.Path : path);
         
         if (dev == null) {
            
            Console.WriteLine ("NULL");
            
            return;
         }
         
         Console.WriteLine ("IP: {0}", dev.DeviceIP );
         Console.WriteLine ("RAPI address: {0}:{1}", dev.DeviceRapiIP, dev.DeviceRapiPort );
         Console.WriteLine ("Id: {0}", dev.Id);
         Console.WriteLine ("Current partner id: {0}", dev.CurrentPartnerId);
         Console.WriteLine ("OS Version: {0}.{1}.{2}", dev.OsVersionMajor, dev.OsVersionMinor, dev.Version);
         Console.WriteLine ("CPU type: {0}", dev.CpuType);
         Console.WriteLine ("Name: \"{0}\"", dev.Name);
         Console.WriteLine ("Platform name: \"{0}\"", dev.PlatformName);
         Console.WriteLine ("Model name: \"{0}\"", dev.ModelName);
      }
      
      public static void DeviceConnectedEvent (string device_path)
      {
         Device device = Manager [device_path];
         
         PrintDevice (device, device_path, "Connected");
      }

      public static void DeviceDisconnectedEvent (string device_path)
      {
         PrintDevice (Manager [device_path], device_path, "Disconnected");
      }

      public static void DevicePasswordRequestedEvent (string device_path)
      {
         PrintDevice (Manager [device_path], device_path, "Password Requested");
      }
      
      public static void DeviceConnectionEstablishedEvent (string device_path)
      {
         PrintDevice (Manager [device_path], device_path, "Connection Established");
      }
      
      public static void DeviceWrongPasswordEvent (string device_path)
      {
         PrintDevice (Manager [device_path], device_path, "Wrong Password");
      }
      
      public static int Main (string[] args)
      {
         try {
            Utilities.SetProcessName (Defines.Package);
         }
         catch {
         }
         
         Catalog.Init (Defines.Package, Defines.LocaleDir);

         if (!org.wimo.Tools.DBusInit ())
            return -1;

         Opt = new DaemonOptions ();
                                                                  
         Opt.ProcessArgs (args);
         
         Manager = new DeviceManager ();

         if (Opt.Verbose) {
            
            Manager.DeviceConnected += DeviceConnectedEvent;
            Manager.DeviceDisconnected += DeviceDisconnectedEvent;
            Manager.DevicePasswordRequested += DevicePasswordRequestedEvent;
            Manager.DeviceConnectionEstablished += DeviceConnectionEstablishedEvent;
            Manager.DeviceWrongPassword += DeviceWrongPasswordEvent;
         }
         
         try {
            Manager.Register ();
         }
         catch (Exception e) {
            
            Console.Error.WriteLine ("Failed to register DeviceManager interface.\n\nException:\n{0}", e.Message);
            
            return -2;
         }
         
         new Detector ();
         
         try {
            new DccmServer ();
         }
         catch (Exception e) {
            
            Console.Error.WriteLine (Catalog.GetString ("Failed to initialize DCCM server socket. {0}"), e.Message);
            
            return -3;
         }
         
         try {
            new RapiServer ();
         }
         catch (Exception e) {
            
            Console.Error.WriteLine (Catalog.GetString ("Failed to initialize RAPI server socket. {0}"), e.Message);
            
            return -4;
         }
         
         if (Opt.Verbose)
            Console.WriteLine (Catalog.GetString ("Running") + (Opt.Debug ? " (debug)" : ""));

         while (true)
            Bus.System.Iterate ();
      }
   }
}